@@ -11,16 +11,17 @@
 //!     let nodes = vec![
 //!         NodeConstructor::new(0, Default::default())
 //!             .with_title(|ui| ui.label("Example Node A"))
-//!             .with_input_attribute(0, Defaut::default(), |ui| ui.label("Input"))
+//!             .with_input_attribute(0, Default::default(), |ui| ui.label("Input"))
 //!             .with_static_attribute(1, |ui| ui.label("Can't Connect to Me"))
-//!             .with_output_attribute(2, Defaut::default(), |ui| ui.label("Output")),
+//!             .with_output_attribute(2, Default::default(), |ui| ui.label("Output")),
 //!         NodeConstructor::new(1, Default::default())
 //!             .with_title(|ui| ui.label("Example Node B"))
 //!             .with_static_attribute(3, |ui| ui.label("Can't Connect to Me"))
-//!             .with_output_attribute(4, Defaut::default(), |ui| ui.label("Output"))
-//!             .with_input_attribute(5, Defaut::default(), |ui| ui.label("Input"))
+//!             .with_output_attribute(4, Default::default(), |ui| ui.label("Output"))
+//!             .with_input_attribute(5, Default::default(), |ui| ui.label("Input"))
 //!     ];
-//!     
+//! 
+//!     // add them to the ui
 //!     ctx.show(
 //!         nodes,
 //!         links.iter().enumerate().map(|(i, (start, end))| (i, *start, *end, LinkArgs::default())),
@@ -33,7 +34,7 @@
 //!     }
 //!
 //!     // add created links
-//!     if let Some((start, end, _)) = self.ctx.link_created() {
+//!     if let Some((start, end, _)) = ctx.link_created() {
 //!         links.push((start, end))
 //!     }
 //! }
@@ -441,15 +442,11 @@ impl Context {
 }
 
 impl Context {
-    fn add_node<'a>(&mut self, idx: usize, NodeConstructor{id, title, attributes, mut pos, args}: NodeConstructor<'a>, ui: &mut egui::Ui) {
-        pos = pos.map(|x| self.screen_space_to_grid_space(x));
+    fn add_node<'a>(&mut self, idx: usize, NodeConstructor{id, title, attributes, pos: _, args}: NodeConstructor<'a>, ui: &mut egui::Ui) {
         let node = &mut self.nodes.pool[idx];
         self.style.format_node(node, args);
         node.background_shape.replace(ui.painter().add(egui::Shape::Noop));
         node.id = id;
-        if let Some(pos) = pos {
-            node.origin = pos;
-        }
         let node_origin = node.origin;
         let node_size = node.size;
         let title_space = node.layout_style.padding.y;
@@ -463,20 +460,22 @@ impl Context {
                 title_info.replace((titlebar_shape, title_bar_content_rect));
                 ui.add_space(title_space);
             }
+            let outline_shape = ui.painter().add(egui::Shape::Noop);
             for (id, kind, args, attribute) in attributes {
                 let response = ui.allocate_ui(ui.available_size(), attribute);
                 let shape = ui.painter().add(egui::Shape::Noop);
                 let response = response.response.union(response.inner);
                 self.add_attribute(id, kind, args, response, idx, shape);
             }
-            title_info
+            (title_info, outline_shape)
         });
         let node = &mut self.nodes.pool[idx];
-        if let Some((titlebar_shape, title_bar_content_rect)) = response.inner {
+        let (title_info, outline_shape) = response.inner;
+        if let Some((titlebar_shape, title_bar_content_rect)) = title_info {
             node.titlebar_shape.replace(titlebar_shape);
             node.title_bar_content_rect = title_bar_content_rect;
         }
-        node.outline_shape.replace(ui.painter().add(egui::Shape::Noop));
+        node.outline_shape.replace(outline_shape);
         node.rect = response.response.rect.expand2(node.layout_style.padding);
         if response.response.hovered() {
             self.node_indices_overlapping_with_mouse.push(idx);
@@ -598,8 +597,8 @@ impl Context {
         for depth_idx in 0..(depth_stack.len() - 1) {
     
             let node_below = &self.nodes.pool[depth_stack[depth_idx]];
-            for next_depth_idx in (depth_idx + 1)..(depth_stack.len()) {
-                let rect_above = self.nodes.pool[depth_stack[next_depth_idx]].rect;
+            for next_depth in &depth_stack[(depth_idx + 1)..(depth_stack.len())] {
+                let rect_above = self.nodes.pool[*next_depth].rect;
                 for idx in node_below.pin_indices.iter() {
                     let pin_pos = self.pins.pool[*idx].pos;
                     if rect_above.contains(pin_pos) {
@@ -687,10 +686,8 @@ impl Context {
         let link_shape = link.shape.take().unwrap();
         let link_hovered = self.hovered_link_idx == Some(link_idx) && self.click_interaction_type != ClickInteractionType::BoxSelection;
 
-        if link_hovered {
-            if self.left_mouse_clicked {
-                self.begin_link_interaction(link_idx);
-            }
+        if link_hovered && self.left_mouse_clicked {
+            self.begin_link_interaction(link_idx);
         }
 
         if self.deleted_link_idx == Some(link_idx) {
@@ -738,10 +735,8 @@ impl Context {
             self.draw_pin(pin_idx, ui);
         }
 
-        if node_hovered {
-            if self.left_mouse_clicked && self.interactive_node_index != Some(node_idx) {
-                self.begin_node_selection(node_idx);
-            }
+        if node_hovered && self.left_mouse_clicked && self.interactive_node_index != Some(node_idx) {
+            self.begin_node_selection(node_idx);
         }
     } 
 
@@ -830,10 +825,8 @@ impl Context {
 
         self.selected_node_indices.clear();
         for (idx, node) in self.nodes.pool.iter().enumerate() {
-            if self.nodes.in_use[idx] {
-                if box_rect.intersects(node.rect) {
-                    self.selected_node_indices.push(idx);
-                }
+            if self.nodes.in_use[idx] && box_rect.intersects(node.rect) {
+                self.selected_node_indices.push(idx);
             }
         }
 
@@ -872,7 +865,7 @@ impl Context {
             }
 
             let link_data = LinkBezierData::get_link_renderable(*start, *end, start_type, self.style.link_line_segments_per_length);
-            return link_data.rectangle_overlaps_bezier(&rect);
+            return link_data.rectangle_overlaps_bezier(rect);
         }
         false
     }
@@ -993,19 +986,17 @@ impl Context {
                 self.begin_link_detach(idx, self.hovered_pin_index.unwrap());
                 self.click_interaction_state.link_creation.link_creation_type = LinkCreationType::FromDetach;
             }
+        } else if self.link_detatch_with_modifier_click {
+            let link = &self.links.pool[idx];
+            let start_pin = &self.pins.pool[link.start_pin_index];
+            let end_pin = &self.pins.pool[link.end_pin_index];
+            let dist_to_start = start_pin.pos.distance(self.mouse_pos);
+            let dist_to_end = end_pin.pos.distance(self.mouse_pos);
+            let closest_pin_idx = if dist_to_start < dist_to_end { link.start_pin_index } else {link.end_pin_index};
+            self.click_interaction_type = ClickInteractionType::LinkCreation;
+            self.begin_link_detach(idx, closest_pin_idx);
         } else {
-            if self.link_detatch_with_modifier_click {
-                let link = &self.links.pool[idx];
-                let start_pin = &self.pins.pool[link.start_pin_index];
-                let end_pin = &self.pins.pool[link.end_pin_index];
-                let dist_to_start = start_pin.pos.distance(self.mouse_pos);
-                let dist_to_end = end_pin.pos.distance(self.mouse_pos);
-                let closest_pin_idx = if dist_to_start < dist_to_end { link.start_pin_index } else {link.end_pin_index};
-                self.click_interaction_type = ClickInteractionType::LinkCreation;
-                self.begin_link_detach(idx, closest_pin_idx);
-            } else {
-                self.begin_link_selection(idx);
-            }
+            self.begin_link_selection(idx);
         }
     }
 
@@ -1171,7 +1162,7 @@ struct ObjectPool<T> {
 
 impl<T> ObjectPool<T> {
     fn find(&self, id: usize) -> Option<usize> {
-        self.map.get(&id).map(|x| *x)
+        self.map.get(&id).copied()
     }
     fn reset(&mut self) {
         self.in_use.iter_mut().for_each(|x| *x = false);
@@ -1191,7 +1182,7 @@ impl<T: Id> ObjectPool<T> {
 
     fn find_or_create_index(&mut self, id: usize) -> usize {
         let index = {
-            if let Some(index) = self.map.get(&id).map(|x| *x) {
+            if let Some(index) = self.find(id) {
                 index
             } else {
                 let index = if let Some(index) = self.free.pop() {
@@ -1230,12 +1221,12 @@ impl Context {
 
     fn node_pool_find_or_create_index(&mut self, id: usize, origin: Option<egui::Pos2>) -> usize {
         let index = {
-            if let Some(index) = self.nodes.map.get(&id).map(|x| *x) {
+            if let Some(index) = self.nodes.find(id) {
                 index
             } else {
                 let mut new_node = NodeData::new(id);
                 if let Some(origin) = origin {
-                    new_node.origin = origin;
+                    new_node.origin = self.screen_space_to_grid_space(origin);
                 }
                 let index = if let Some(index) = self.nodes.free.pop() {
                     self.nodes.pool[index] = new_node;
